/******************************************************************************
*
* project name:   TICT Tutorial S1P6
* file name:      scroller.c
* initial date:   19/12/2000
* authors:        thomas.nussbaumer@gmx.net
*                 lionel_debroux@yahoo.fr
*
* description:    Simple GrayScale Text Scroller with a Background Image
*
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
* NOTE: depends on TIGCC 0.95 and later to compile
* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*
* Note: in the early betas of TIGCC 0.95, -mpcrel is buggy: it generates bsr.l,
* which is incorrect on plain 68000...
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
* v2.00: 30/10/2003
*        Modifications by Lionel Debroux (unordered list):
*          - Introduced a compressed image and the corresponding decompression
*            routine. This reduces the size by 600 bytes (more than 10%).
*            It can be disabled/enabled undefining/defining USE_COMPRESSED_IMAGE.
*          - Introduced doublebuffering for flicker-free grayscale.
*            It can be disabled/enabled undefining/defining USE_DOUBLEBUFFERING.
*          - Speedups: assembly, inline loops, fast method for drawing strings...
*          - Size optimization: turned a huge array of unsigned char * into
*            assembly. There are no more relocations for demotext, the size is
*            reduced by ~350 bytes...
*        Compare this file and "scroller_old.c"...
*
* v1.00: 19/12/2000
*        Initial version
*
******************************************************************************/

// Uncomment either USE_TI89 or (USE_TI92P and USE_V200) to build the project.
// #define USE_TI89
// #define USE_TI92P
// #define USE_V200

#if    defined(USE_TI89) && defined(USE_TI92P) \
    || defined(USE_TI89) && defined(USE_V200)
  #error Cannot produce both versions at once
#elif defined(USE_TI89)
  #define C89_92(x,y) (x)
#elif (defined(USE_TI92P) || defined(USE_V200))
  #define USE_TI92P
  #define USE_V200
  #define C89_92(x,y) (y)
#else
  #error Please specify either -DUSE_TI89 or -DUSE_TI92P or -DUSE_V200 \
  on command line
#endif

#define OPTIMIZE_ROM_CALLS
#define NO_EXIT_SUPPORT
#define MIN_AMS 101

// _nostub comment format (new in TIGCC 0.95).
#define COMMENT_STRING "Tutorial S1P6 by TICT"
#define COMMENT_VERSION_NUMBER 2,0,0,0

#include <tigcclib.h>


// On TI-92+ and V200 the image is approximatively centered.
// Indeed, the address where to write the data MUST be even (we transfer longs).
// The correct offset (image correctly centered) is 30*14+40/8.
#define offset C89_92(0,30*14+32/8)  // On the TI92p and V200 we center the image.

// Not using doublebuffering is bigger than using it !
#define USE_DOUBLEBUFFERING
#define USE_COMPRESSED_IMAGE

//-----------------------------------------------------------------------------
// In this tutorial I use a blackwhite bitmap of 160x100 pixels
// as background (converted using ImageStudio and TTBin2Hex).
//
// The compressed image can be compressed by a routine whose implementation 
// is left to the reader...
//-----------------------------------------------------------------------------

#ifdef USE_COMPRESSED_IMAGE

static unsigned char data[] = {
15,0x93,0xF6,0xFD,19,0x93,
0xAD,0x7F,16,0x93,0xF8,0x73,18,0x93,
0xF0,0x0B,0xFE,0xDF,16,0x93,0xE0,0x24,
0xFF,0xDF,16,0x93,0x9C,0xD0,0xAC,0x87,16,0x93,
0xF2,0x7E,0x60,0xE1,16,0x93,0xE6,0x6B,0x13,0x88,12,0x93,
0xFE,3,0x93,0xA2,0x69,0xE1,0xFD,0x4F,11,0x93,0xFE,3,0x93,
0x71,0xD8,0x60,0xC9,0x9F,14,0x93,0xFE,0x1E,0x55,0x98,0x76,0x8F,14,0x93,
0xFE,0x89,0x19,0x18,0x77,0x13,15,0x93,0x7F,0xCD,
0x68,0x7F,0xC3,12,0x93,0xFE,0xFF,0xFF,0xBF,0x9B,0x80,0xEE,0x41,15,0x93,
0xA1,0xDA,0xC0,0x10,0x88,15,0x93,0xFA,0x76,0xC0,0x70,0x0D,15,0x93,
0xFB,0xF3,0x00,0x70,0x13,16,0x93,
0x90,0x00,0x10,0x35,15,0x93,0xDF,0x41,0x40,0x00,0x46,16,0x93,
0x97,0xC0,0x00,0x43,15,0x93,0xE9,0x7B,
0x00,0x02,0x2D,15,0x93,0xE2,0x62,0x58,0x01,0x87,0xB7,14,0x93,
0xFE,0xDE,0x70,0x01,0x21,0xB7,14,0x93,0xF9,0x9D,0xF8,0xA2,0xCD,0x7F,14,0x93,
0xF2,0xB8,0xCB,0x7A,0x3C,0xDF,5,0x93,0xAE,8,0x93,
0xFE,0xF8,0x1F,0x72,0x16,0x5B,5,0x93,0x06,0x3F,8,0x93,0xE6,0x63,0x72,0x73,0x7F,4,0x93,
0xFC,0xE6,0x67,7,0x93,0xFE,0xBB,0xB0,0x20,0xC1,0xBF,4,0x93,0xF2,0x00,0x07,5,0x93,0xF7,0xFF,0xFC,0x8A,
0xB7,0x23,0x79,0x5F,4,0x93,0xF8,0x00,0x01,8,0x93,0xC3,0xF7,0x83,0xFF,0x07,4,0x93,0xD8,0x00,0x00,0x7F,
6,0x93,0xFE,0xF3,0xFF,0x62,0xE7,0x4B,4,0x93,0x90,0x00,0x00,0xF0,0x01,6,0x93,0xDE,0x38,0x1A,0xCE,0xCF,
3,0x93,0xFD,0x80,0x00,0x00,0x80,0x00,0x27,6,0x93,0x90,0x15,0x24,0x6F,3,0x93,0xFE,0x20,0x00,0x01,0x0F,0x80,0x01,6,0x93,
0xE2,0xE5,0xD9,0xE3,3,0x93,0xFA,0xC0,0x00,0x02,0x0F,0xFF,0xAC,0x2F,5,0x93,0x9F,0x14,0x0F,0xA3,4,0x93,
0x00,0x00,0x00,0x3F,0xFF,0xFF,0xC5,6,0x93,0xB0,0x3F,0xF7,3,0x93,0xFD,0x80,0x00,0x00,3,0x93,0xF0,0x7F,4,0x93,
0xFE,0x03,0xFF,0xF7,4,0x93,0x00,0x00,
0x03,3,0x93,0xFA,0x3F,4,0x93,0xFC,0xCF,0xFF,0xF7,3,0x93,0xFD,0x80,0x00,0x1F,4,0x93,
0x07,4,0x93,0xFE,0x79,0xFF,0xF7,3,0x93,0xFD,0x80,0x00,5,0x93,0xA7,7,0x93,0xF7,
4,0x93,0x80,0x01,5,0x93,0xF9,7,0x93,0xDF,3,0x93,0xFE,0x80,0x07,5,0x93,0xFC,
0x7F,10,0x93,0xFE,0x01,0x7F,6,0x93,0x3F,6,0x93,0xFE,3,0x93,0xFE,
0x07,7,0x93,0x9F,6,0x93,0xFE,3,0x93,0xFC,0x3F,7,0x93,0xC7,10,0x93,
0xB4,0x7F,7,0x93,0xC3,0xFF,0xFE,4,0x93,0xFE,3,0x93,0xF8,8,0x93,
0xF1,10,0x93,0xE0,8,0x93,0xF9,9,0x93,
0xFE,0xE1,8,0x93,0xFC,10,0x93,0xC3,8,0x93,
0xFC,0xFF,0xFF,0xFE,0xFF,0xFF,0x7F,0x7F,0xFF,0xFF,0xFD,0x87,9,0x93,0x3F,6,0x93,0x3F,0xFF,0xFF,0x9F,
9,0x93,0x1F,6,0x93,0xBF,0xFF,0xFB,0x9F,9,0x93,0x9F,6,0x93,
0x9F,0xFF,0xFF,0x3F,9,0x93,0xCF,6,0x93,0x97,0xFF,0xFE,0x7F,9,0x93,
0xCF,6,0x93,0xC7,0xFF,0xFE,0x3F,9,0x93,0xCF,0xFF,0xFF,0xEF,3,0x93,
0xC7,0xFF,0xFC,0x7F,9,0x93,0xE7,0xFF,0xFF,0xEF,3,0x93,0x90,0xFF,0xFC,10,0x93,
0xE3,6,0x93,0x88,0x7F,0xFC,0x7F,9,0x93,0xF3,6,0x93,0xC8,0x3F,0xFD,10,0x93,
0xF3,6,0x93,0x46,0xCF,0xF9,10,0x93,0xF9,6,0x93,
0x46,0x7F,0xF9,10,0x93,0xFD,6,0x93,0x63,0x7F,0xF1,10,0x93,
0xF8,6,0x93,0xF3,0xC7,0xF3,10,0x93,0xFC,6,0x93,
0xB3,0xE7,0xF1,10,0x93,0xFA,0x7F,0xFF,0x67,0xF9,0xD1,0xCF,0x11,0xFB,0xF3,10,0x93,
0xED,0x3F,0xFE,0xDF,0xF0,0x91,0xEF,0x19,0xF9,0xF5,10,0x93,0xC9,0xBF,0xFE,0xD7,0xF2,0x73,0xFF,0x98,0xC1,0xF5,0xFF,
0xFF,0xF7,4,0x93,0xF9,0xDF,0xFF,0xD9,0xDF,0xFC,0xCF,0xF7,0xC1,0xFF,0x98,0xFC,0xBD,0x83,0xC5,0x24,0x0B,0x20,0x3C,0xD2,0x62,0x15,0x1D,0xB9,0xF4,0x70,
0x97,0xE1,0xBC,0xFF,0xA9,0x7C,0xF9,0x81,0xB6,0x2C,0x90,0xC8,0x1B,0x87,0xA3,0xE0,0x06,0x31,0xF0,0xE1,0x27,0xE1,0xFE,0xFE,0x7C,0xCE,0x21,0x26,0x89,0x0E,0x36,0x92,
0x6C,0xA1,0x1C,0x63,0xEE,0x33,0xF6,0x06,0x2F,0xE5,0xEF,0xC1,0xFC,0xA5,0x0B,0x36,0x5C,0x0B,0xEE,0x37,0x6F,0xA6,0x7C,0xAB,0x9C,0xA4,0xC1,0x76,0x2F,0xF1,0xFF,0xD9,
0xEC,0xC1,0xD5,0xC9,0xFC,0xE7,0xCF,0xF0,0x20,0x1B,0xC3,0xAA,0x32,0x4C,0x0F,0xFC,0x6F,0xF5,0xFF,0xAA,0xF8,0xAB,0x75,0xC6,0x90,0x7F,0x97,0xAF,0xEF,0xC9,0xC3,0xE1,
0x7E,0x70,0x13,0x7D,0xA6,0x35,0xFF,0x98,0x78,0xC1,0x39,0x67,0x0D,0x97,0xB7,0x85,0xBF,0xFE,0xD8,0x29,0xC2,0xCF,0xDE,0x38,0xFA,0x21,0xFF,0x39,0xF8,0x7D,0x51,0x7E,
0x7B,0x77,0x77,0xCA,0x3F,0xFF,0x88,0xCC,0x78,0x3F,0xDE,0x30,0xF2,0xC1,0xFF,0xB9,0xFE,0x6D,0x16,0xE3,0x3D,0xBF,0xFA,0xFF,0x9F,0xFF,0x29,0xC3,0x92,0x01,0xDE,0x71,
0xF6,0x71,0xFF,0x1D,0xEA,0x6F,0x17,0xD6,0xB9,0xBF,0x7C,0xA4,0x8F,0xFF,0xF7,0xDA,0x80,0x04,0x5B,0xFF,0xE5,0x61,0xFE,0x36,0xFA,0x4D,0x85,0xAC,0xFB,0xFF,0xFE,0x60,
0x1F,0xFF,0xFF,0xF9,0xAC,0x17,0xE9,0x1F,0xCB,0xE7,0xFF,0x3D,0xFD,0xBD,0xCF,0xC4,0xF5,0xE7,0xFF,0x3C,0x2F,0xFF,0xFF,0xFB,0xB8,0x1F,0xD8,0xCE,0x48,0x6F,0xFF,0x77,
0xF8,0x87,0x4E,0xE8,0x73,0xFF,0xFF,0xD9,0xEF,3,0x93,0xEB,0x5F,0x88,0xDC,0x9A,0xEF,0xFF,0x1F,0xFB,0x67,0xCB,0x19,0x22,0xF7,0xFF,0xDB,0x1F,3,0x93,
0xF7,0xFF,0x9C,0xF0,0xB4,0xEF,0xFF,0x5F,0xFE,0x47,0xED,0x71,0x41,0xF3,0xFF,0xEA,0x1F,0xE0,4,0x93,0xFC,0x19,0xA7,0x8C,0xFF,0x1F,0xFE,0x77,0x8D,0x17,
0xD9,0xFB,0x8F,0xED,0xDF,0x18,0x7F,3,0x93,0xFA,0x71,0xCF,0xFD,0xFF,0x1F,0xFF,0x67,0x8D,0x10,0x33,0xFE,0x77,0xFE,0xDD,0x4E,0x6E,0x3F,0xFF,0xFF,0xFC,0x05,
0x4C,0xF1,0xFF,0x5F,0xFE,0xF7,0xCB,0x84,0xE7,0xFE,0x7B,0xFB,0x3C,0x4F,0xE0,0x1F,0xFF,0xFF,0xFA,0xC4,0x5E,0x01,0xCF,0x1F,0xFF,0xFF,0xA9,0x94,0x81,0xFE,0x7F,0xFB,
0xF3,0x1F,0xC9,0xB2,0x1F,0xFF,0xFE,0x85,0xC8,0x0B,0xCF,0x3F,0xFF,0xBF,0xAE,0x47,0x0F,0xFE,0x79,0xF0,0x27,0x4F,0x53,0xF0,0x21,0xFF,0xFE,0x94,0x12,0x01,0xC7,0x1F,
0xFF,0x87,0xAF,0xCF,0x5B,0xFC,0xFC,0x18,0x67,0x8F,0x0F,0x71,0xEC,0x3F,0xFF,0xF9,0x40,0xC1,0x03,0x7F,0xFF,0x9B,0x85,0xCF,0x37,0xFC,0xFE,0xF3,0xA7,0x2B,0x71,0xF0,
0x50,0x1F,0xFF,0xB3,0xF4,0x77,0x18,0xF0,0xFF,0x11,0xD7,0x4E,0xEF,0xF9,0x7F,0xFB,0xEC,0xF5,0xE0,0xFE,0x33,0xDF,0xFF,0xBB,0xF0,0xEF,0x74,0x00,0xFF,0x19,0x8F,0x82,
0x7F,0xF4,0x7D,0x6F,0xFC,0x3D,0x06,0x7F,0x9B,0x5F,0xFF,0xF3,0xF3,0xFE,0x7C,0x7F,0xFF,0x98,0xE7,0xCF,0xEF,0xE5,0x7F,0x6F,0xC1,0xBB,0xFF,0xEF,0xB0,0xC2,0xFF,0xFB,
0xF2,0xFE,0xFD,0x6F,0xFF,0x99,0xEF,0x98,0x5E,0xEF,0xBF,0x7F,0x9B,0xDF,0xFF,0xC0,0x24,0xF2,0x3F,0xF9,0xF0,0xFE,0x0F,0x0F,0xFF,0xA8,0xEF,0xA5,0x8D,0xFF,0xFF,0xBE,
0x50,0xFF,0xFF,0xE0,0x41,0xF4,0xE1,0xFB,0xF7,0xFF,0x73,0x1F,0xFF,0xFC,0xFF,0xA9,0xD8,0xFF,0xFE,0x60,0x56,3,0x93,0x4F,0xF0,0xF8,0x3B,0xF7,0xFF,0x7B,0x7F,
0xFF,0xBC,0x6F,0x87,0xFB,0x7E,0x22,0xA0,0x06,3,0x93,0x5C,0x26,0x92,0x1F,5,0x93,0xEE,0xFF,0xC7,0x9F,0xF0,0x60,0xFD,0x4F,3,0x93,
0x70,0x60,0x72,0x5F,5,0x93,0x78,0xFF,0x9F,0xC7,0xF0,0x44,0xF4,0xC1,3,0x93,0xF1,0x47,0x9B,0x5F,5,0x93,0xF8,0xFF,0xAF,
0xC7,0xFF,0x8E,0xF4,0xA1,0xFF,0xFF,0xFC,0x7F,0x8E,0xB8,0xEF,4,0x93,
// end marker
0x93,0x93
};

#else

static unsigned long data[] = {
0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFF6,0xFDFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFFFF,0xAD7FFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFF873,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFFFF00B,0xFEDFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFE024,
0xFFDFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFF9CD0,0xAC87FFFF,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFF27E,0x60E1FFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFE66B,0x1388FFFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFFFFEFF,0xFFFFA269,0xE1FD4FFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFEFF,
0xFFFF71D8,0x60C99FFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFE1E55,0x98768FFF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFFE8919,0x187713FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFF7FCD,
0x687FC3FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFE,0xFFFFBF9B,0x80EE41FF,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFA1DA,0xC01088FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFA76,0xC0700DFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFBF3,0x007013FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFF90,0x001035FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFDF41,0x400046FF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFFFFF97,0xC00043FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFE97B,
0x00022DFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFE262,0x580187B7,0xFFFFFFFF,0xFFFFFFFF,
0xFFFFFFFF,0xFFFFFEDE,0x700121B7,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFF99D,0xF8A2CD7F,
0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFF2B8,0xCB7A3CDF,0xFFFFFFFF,0xFFAEFFFF,0xFFFFFFFF,
0xFFFFFEF8,0x1F72165B,0xFFFFFFFF,0xFF063FFF,0xFFFFFFFF,0xFFFFFFE6,0x6372737F,0xFFFFFFFF,
0xFCE667FF,0xFFFFFFFF,0xFFFFFEBB,0xB020C1BF,0xFFFFFFFF,0xF20007FF,0xFFFFFFFF,0xF7FFFC8A,
0xB723795F,0xFFFFFFFF,0xF80001FF,0xFFFFFFFF,0xFFFFFFC3,0xF783FF07,0xFFFFFFFF,0xD800007F,
0xFFFFFFFF,0xFFFFFEF3,0xFF62E74B,0xFFFFFFFF,0x900000F0,0x01FFFFFF,0xFFFFFFDE,0x381ACECF,
0xFFFFFFFD,0x80000080,0x0027FFFF,0xFFFFFFFF,0x9015246F,0xFFFFFFFE,0x2000010F,0x8001FFFF,
0xFFFFFFFF,0xE2E5D9E3,0xFFFFFFFA,0xC000020F,0xFFAC2FFF,0xFFFFFFFF,0x9F140FA3,0xFFFFFFFF,
0x0000003F,0xFFFFC5FF,0xFFFFFFFF,0xFFB03FF7,0xFFFFFFFD,0x800000FF,0xFFFFF07F,0xFFFFFFFF,
0xFE03FFF7,0xFFFFFFFF,0x000003FF,0xFFFFFA3F,0xFFFFFFFF,0xFCCFFFF7,0xFFFFFFFD,0x80001FFF,
0xFFFFFF07,0xFFFFFFFF,0xFE79FFF7,0xFFFFFFFD,0x8000FFFF,0xFFFFFFA7,0xFFFFFFFF,0xFFFFFFF7,
0xFFFFFFFF,0x8001FFFF,0xFFFFFFF9,0xFFFFFFFF,0xFFFFFFDF,0xFFFFFFFE,0x8007FFFF,0xFFFFFFFC,
0x7FFFFFFF,0xFFFFFFFF,0xFFFFFFFE,0x017FFFFF,0xFFFFFFFF,0x3FFFFFFF,0xFFFFFFFE,0xFFFFFFFE,
0x07FFFFFF,0xFFFFFFFF,0x9FFFFFFF,0xFFFFFFFE,0xFFFFFFFC,0x3FFFFFFF,0xFFFFFFFF,0xC7FFFFFF,
0xFFFFFFFF,0xFFFFFFB4,0x7FFFFFFF,0xFFFFFFFF,0xC3FFFEFF,0xFFFFFFFE,0xFFFFFFF8,0xFFFFFFFF,
0xFFFFFFFF,0xF1FFFFFF,0xFFFFFFFF,0xFFFFFFE0,0xFFFFFFFF,0xFFFFFFFF,0xF9FFFFFF,0xFFFFFFFF,
0xFFFFFEE1,0xFFFFFFFF,0xFFFFFFFF,0xFCFFFFFF,0xFFFFFFFF,0xFFFFFFC3,0xFFFFFFFF,0xFFFFFFFF,
0xFCFFFFFE,0xFFFF7F7F,0xFFFFFD87,0xFFFFFFFF,0xFFFFFFFF,0xFF3FFFFF,0xFFFFFFFF,0x3FFFFF9F,
0xFFFFFFFF,0xFFFFFFFF,0xFF1FFFFF,0xFFFFFFFF,0xBFFFFB9F,0xFFFFFFFF,0xFFFFFFFF,0xFF9FFFFF,
0xFFFFFFFF,0x9FFFFF3F,0xFFFFFFFF,0xFFFFFFFF,0xFFCFFFFF,0xFFFFFFFF,0x97FFFE7F,0xFFFFFFFF,
0xFFFFFFFF,0xFFCFFFFF,0xFFFFFFFF,0xC7FFFE3F,0xFFFFFFFF,0xFFFFFFFF,0xFFCFFFFF,0xEFFFFFFF,
0xC7FFFC7F,0xFFFFFFFF,0xFFFFFFFF,0xFFE7FFFF,0xEFFFFFFF,0x90FFFCFF,0xFFFFFFFF,0xFFFFFFFF,
0xFFE3FFFF,0xFFFFFFFF,0x887FFC7F,0xFFFFFFFF,0xFFFFFFFF,0xFFF3FFFF,0xFFFFFFFF,0xC83FFDFF,
0xFFFFFFFF,0xFFFFFFFF,0xFFF3FFFF,0xFFFFFFFF,0x46CFF9FF,0xFFFFFFFF,0xFFFFFFFF,0xFFF9FFFF,
0xFFFFFFFF,0x467FF9FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFDFFFF,0xFFFFFFFF,0x637FF1FF,0xFFFFFFFF,
0xFFFFFFFF,0xFFF8FFFF,0xFFFFFFFF,0xF3C7F3FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFCFFFF,0xFFFFFFFF,
0xB3E7F1FF,0xFFFFFFFF,0xFFFFFFFF,0xFFFA7FFF,0x67F9D1CF,0x11FBF3FF,0xFFFFFFFF,0xFFFFFFFF,
0xFFED3FFE,0xDFF091EF,0x19F9F5FF,0xFFFFFFFF,0xFFFFFFFF,0xFFC9BFFE,0xD7F273FF,0x98C1F5FF,
0xFFF7FFFF,0xFFFFF9DF,0xFFD9DFFC,0xCFF7C1FF,0x98FCBD83,0xC5240B20,0x3CD26215,0x1DB9F470,
0x97E1BCFF,0xA97CF981,0xB62C90C8,0x1B87A3E0,0x0631F0E1,0x27E1FEFE,0x7CCE2126,0x890E3692,
0x6CA11C63,0xEE33F606,0x2FE5EFC1,0xFCA50B36,0x5C0BEE37,0x6FA67CAB,0x9CA4C176,0x2FF1FFD9,
0xECC1D5C9,0xFCE7CFF0,0x201BC3AA,0x324C0FFC,0x6FF5FFAA,0xF8AB75C6,0x907F97AF,0xEFC9C3E1,
0x7E70137D,0xA635FF98,0x78C13967,0x0D97B785,0xBFFED829,0xC2CFDE38,0xFA21FF39,0xF87D517E,
0x7B7777CA,0x3FFF88CC,0x783FDE30,0xF2C1FFB9,0xFE6D16E3,0x3DBFFAFF,0x9FFF29C3,0x9201DE71,
0xF671FF1D,0xEA6F17D6,0xB9BF7CA4,0x8FFFF7DA,0x80045BFF,0xE561FE36,0xFA4D85AC,0xFBFFFE60,
0x1FFFFFF9,0xAC17E91F,0xCBE7FF3D,0xFDBDCFC4,0xF5E7FF3C,0x2FFFFFFB,0xB81FD8CE,0x486FFF77,
0xF8874EE8,0x73FFFFD9,0xEFFFFFFF,0xEB5F88DC,0x9AEFFF1F,0xFB67CB19,0x22F7FFDB,0x1FFFFFFF,
0xF7FF9CF0,0xB4EFFF5F,0xFE47ED71,0x41F3FFEA,0x1FE0FFFF,0xFFFFFC19,0xA78CFF1F,0xFE778D17,
0xD9FB8FED,0xDF187FFF,0xFFFFFA71,0xCFFDFF1F,0xFF678D10,0x33FE77FE,0xDD4E6E3F,0xFFFFFC05,
0x4CF1FF5F,0xFEF7CB84,0xE7FE7BFB,0x3C4FE01F,0xFFFFFAC4,0x5E01CF1F,0xFFFFA994,0x81FE7FFB,
0xF31FC9B2,0x1FFFFE85,0xC80BCF3F,0xFFBFAE47,0x0FFE79F0,0x274F53F0,0x21FFFE94,0x1201C71F,
0xFF87AFCF,0x5BFCFC18,0x678F0F71,0xEC3FFFF9,0x40C1037F,0xFF9B85CF,0x37FCFEF3,0xA72B71F0,
0x501FFFB3,0xF47718F0,0xFF11D74E,0xEFF97FFB,0xECF5E0FE,0x33DFFFBB,0xF0EF7400,0xFF198F82,
0x7FF47D6F,0xFC3D067F,0x9B5FFFF3,0xF3FE7C7F,0xFF98E7CF,0xEFE57F6F,0xC1BBFFEF,0xB0C2FFFB,
0xF2FEFD6F,0xFF99EF98,0x5EEFBF7F,0x9BDFFFC0,0x24F23FF9,0xF0FE0F0F,0xFFA8EFA5,0x8DFFFFBE,
0x50FFFFE0,0x41F4E1FB,0xF7FF731F,0xFFFCFFA9,0xD8FFFE60,0x56FFFFFF,0x4FF0F83B,0xF7FF7B7F,
0xFFBC6F87,0xFB7E22A0,0x06FFFFFF,0x5C26921F,0xFFFFFFFF,0xFFEEFFC7,0x9FF060FD,0x4FFFFFFF,
0x7060725F,0xFFFFFFFF,0xFF78FF9F,0xC7F044F4,0xC1FFFFFF,0xF1479B5F,0xFFFFFFFF,0xFFF8FFAF,
0xC7FF8EF4,0xA1FFFFFC,0x7F8EB8EF,0xFFFFFFFF};

#endif

typedef struct {
unsigned long* textplane;
void*          savescreen;
void*          image;
volatile unsigned short counter;
#ifdef USE_DOUBLEBUFFERING
void*          dbuffer;
#endif
} scroller_pointers;

scroller_pointers S1P6_DATA;
register scroller_pointers* S1P6_DATA_ptr asm("%a2");

// Be sure to change both defines if you modify their value.
#define ADDITIONAL_LINES  8       // the textplane is oversized, so no
                                  // clipping is necessary
asm(".set ADDITIONAL_LINES,8");

// Be sure to change both defines if you modify their value.
#define LINES_TO_SCROLL     1     // how many lines get scrolled each time
asm(".set LINES_TO_SCROLL,1");

INT_HANDLER save_int_1;           // used to save the old interrupt 1 handler

//=============================================================================
// Our own interrupt 1 handler which only increments the counter.
// Note: currently underoptimized, it will improve in future GCC versions...
//=============================================================================
DEFINE_INT_HANDLER (my_int_1_handler)
{
    // Use the safe way: access to the structure directly instead of 
    // accessing it through S1P6_DATA_ptr, which is not necessarily in a2
    // when the interrupt occurs !
    S1P6_DATA.counter++;
}

extern unsigned char* __F_6x8_Font__;

// NOTE: this is a stripped down version taking only F_6x8 into account !
// What's more, it uses d(pc) mode, forbidden in general purpose code !
extern short SetupCharSet(void);
asm("
	.data
	.even
	.xdef SetupCharSet
SetupCharSet:
| The method is fast in both cases. Therefore, there is no need to suppress interrupts.

	pea (%a2)
	move.l %d3,-(%sp)
	
	movea.l 0xC8.w,%a2

| A cmpi.w would be enough...
	cmpi.l #1000,-4(%a2)
	bgt.s __Prepare_fonts_AMS_2xx__
__Prepare_fonts_AMS_1xx__:
| Get the address of DrawStr.
	movea.l 0x1A9*4(%a2),%a0
	
| Check if we have an old PedroM version (further versions may have other
| signatures ?).
	cmpi.l #0x50656472,-(%a0) | 'Pedr', a signature of PedroM.
	bne.s __The_ROM_we_re_running_on_is_not_PedroM__

| We have PedroM.

| d(pc) mode is forbidden for general purpose code.
	lea __F_6x8_Font__(%pc),%a1
	move.l -8(%a0),(%a1)
	bra.s __Prepare_fonts_end__

__The_ROM_we_re_running_on_is_not_PedroM__:
| Retrieving the addresses of the fonts is easy (but slightly nasty) on AMS 1.xx.

| Catch a pc-relative jump to a subroutine in which are the three addresses we're looking for.
	lea 58+4(%a0),%a0 | Get rid of the cmpi.l #0x50656472,-(%a0) above.
	movea.w (%a0),%a1
	adda.l %a0,%a1
| a1 now contains the absolute address of the subroutine.

| d(pc) mode is forbidden in general purpose code.
	lea __F_6x8_Font__(%pc),%a0

| Setup F_6x8
	move.l 106(%a1),(%a0)

	bra.s __Prepare_fonts_end__

.even
__Prepare_fonts_AMS_2xx__:
| The fonts can be redefined on AMS 2.xx. So, retrieving the addresses of the fonts must be
| performed with FlashApp functions.

	movea.l 0x45D*4(%a2),%a0 | EV_runningApp
	move.w (%a0),%d2
	beq.s __Prepare_fonts_GetAttr__
| OO_GetAppAttr calls OO_GetAttr with the frame of the given application.
| So we are going to catch the frame of the running application, to pass it
| to OO_CondGetAttr.
| We have AMS 2.xx, we can use ROM_CALL 441 'HeapTable' (smaller and faster
| than pushing d3.w on the stack, calling HeapDeref, correcting sp).
        movea.l 0x441*4(%a2),%a0 | HeapTable
	lsl.w #2,%d2
	movea.l 0(%a0,%d2.w),%a0

| The pFrame of an ACB structure is located 20 bytes after the beginning.
	movea.l 20(%a0),%a0
	
	bra.s _Prepare_fonts_GetAttr__

__Prepare_fonts_GetAttr__:
        lea 0xFF000000.l,%a0
_Prepare_fonts_GetAttr__:
	movea.l 0x3FA*4(%a2),%a2 | OO_CondGetAttr
| We use OO_CondGetAttr because it does not throw errors, unlike OO_GetAttr
| (which can call OO_CondGetAttr anyway).

| d(pc) mode is forbidden for general purpose code.
	pea __F_6x8_Font__(%pc)
	pea 0x301.w
	pea (%a0)

	jsr (%a2)
	lea 12(%sp),%sp

	tst.w %d0
	beq.s __Prepare_fonts_problem__

__Prepare_fonts_end__:
	moveq #1,%d0

_Prepare_fonts_end__:
| Don't forget to restore registers...
	move.l (%sp)+,%d3
	move.l (%sp)+,%a2

| This is not an inlined version of the routine...
	rts

__Prepare_fonts_problem__:
	moveq #0,%d0
	bra.s _Prepare_fonts_end__
");

asm("
	.xdef __F_6x8_Font__
	.even
__F_6x8_Font__:
	.long 0
");


// This routine will be faster when it is rewritten in assembly...
/*=============================================================================*/
/* XORs string using own sprites (F_6x8) (much faster than DrawStr)            */
/*=============================================================================*/
static inline void __attribute__((__regparm__)) XORNormalString(short x,const unsigned char* s) {
    unsigned char* sprite;
    long*          addr;
    long*          addr_const = (long *)((unsigned char *)S1P6_DATA_ptr->textplane+2000);
    short          cnt;
    unsigned short c = (unsigned short)*s++;
    unsigned long  buffer;
    unsigned short i;

    while (c) {
        sprite = &(__F_6x8_Font__[(c)<<3]);
        addr   = (long *)((unsigned char *)addr_const+(((x)>>4)+((x)>>4)));
        cnt    = 8-(x&15);

        // Better than an unrolled loop...
        if (cnt < 0) {
            cnt = -cnt;
            for (i=7+1;(i--);) {
                buffer = (unsigned long)(*sprite++)<<16;
                *(long*)addr ^= (long)buffer>>cnt;
                (unsigned char *)addr += 20;
            }
        }
        else {
            for (i=7+1;(i--);) {
                buffer = (unsigned long)(*sprite++)<<16;
                *(long*)addr ^= (long)buffer<<cnt;
                (unsigned char *)addr += 20;
            }
        }

        x += 6;
        c = *s++;
    }
}


//=============================================================================
// allocates space for textplane, charset, screen save and double buffer, and
// installs own interrupt handler
//=============================================================================
static inline short InitScroller(void) {
    if (!(S1P6_DATA_ptr->textplane = (unsigned long*)HeapAllocPtr(
                        20*(100+ADDITIONAL_LINES)
                      + LCD_SIZE
#ifdef USE_DOUBLEBUFFERING
                      + GRAYDBUFFER_SIZE
#endif

#ifdef USE_COMPRESSED_IMAGE
                      + 2000
#endif
                      ))) {
    	return 0;
    }
    
    S1P6_DATA_ptr->savescreen = (void *)((unsigned char *)S1P6_DATA_ptr->textplane+20*(100+ADDITIONAL_LINES));
#ifdef USE_DOUBLEBUFFERING
    S1P6_DATA_ptr->dbuffer = (void *)((unsigned char *)S1P6_DATA_ptr->savescreen+LCD_SIZE);
#endif

#ifdef USE_COMPRESSED_IMAGE
#ifdef USE_DOUBLEBUFFERING
    S1P6_DATA_ptr->image = ((unsigned char *)S1P6_DATA_ptr->dbuffer+GRAYDBUFFER_SIZE);
#else
    S1P6_DATA_ptr->image = ((unsigned char *)S1P6_DATA_ptr->savescreen+LCD_SIZE);
#endif
#else
    S1P6_DATA_ptr->image = data;
#endif

#ifdef USE_DOUBLEBUFFERING
    memset(S1P6_DATA_ptr->dbuffer,0x0000,GRAYDBUFFER_SIZE);
#endif

    SetupCharSet();
    memcpy(S1P6_DATA_ptr->savescreen,LCD_MEM,LCD_SIZE);

#ifdef USE_COMPRESSED_IMAGE
    asm("
    movem.l %%d0-%%d2/%%a0-%%a1,-(%%sp)

| This is correct as long as the compiler puts data in an address register,
| which it is supposed to do...
    lea (%0),%%a0

    movea.l 8(%%a2),%%a1
|    movea.l image(%%pc),%%a1
    moveq #0,%%d0
    moveq #0,%%d1
    moveq #-109,%%d2 | 0x93

_start1_:
    move.b (%%a0)+,%%d0
    move.b (%%a0)+,%%d1

    cmp.b %%d2,%%d1
    beq.s _0x93_

    move.b %%d0,(%%a1)+
    subq.l #1,%%a0
    bra.s _start1_

_0x93_:
    subq.b #1,%%d0

_set_0xFF_:
    st.b (%%a1)+
    dbf %%d0,_set_0xFF_
| We MUST clear d0 ! If we don't, next time, bad things will happen when 
| executing the dbf loop...
    moveq #0,%%d0

| Look for end marker.
    cmp.b (%%a0),%%d2
    bne.s _start1_
    movem.l (%%sp)+,%%d0-%%d2/%%a0-%%a1
    ": : "a"(data));
    // Assembly with C operands is used to make the compiler happy
    // (no warning about data being defined but unused)...
#endif

    // install our own interrupt 1 handler for synchronization and delay
// Generates two relocations...
    register INT_HANDLER * intptr asm("%a0") = &save_int_1;
    *intptr = GetIntVec(AUTO_INT(1));
    S1P6_DATA_ptr->counter = 0;
    SetIntVec(AUTO_INT(1), my_int_1_handler);

    return 1;
}


//=============================================================================
// frees allocated space again and re-installs old interrupt handler
//=============================================================================
static inline void CleanupScroller(void) {
    if (S1P6_DATA_ptr->textplane) {
        memcpy(LCD_MEM,S1P6_DATA_ptr->savescreen,LCD_SIZE);
        HeapFreePtr(S1P6_DATA_ptr->textplane);
        S1P6_DATA_ptr->textplane = NULL;
        SetIntVec(AUTO_INT(1),save_int_1);
    }
}


//=============================================================================
// Blits background planes to grayplanes (combining them with the textplane)
//=============================================================================
static inline void BlitPage(void) {
    unsigned long* backplane_0 = S1P6_DATA_ptr->image;
    unsigned long* tmptext     = S1P6_DATA_ptr->textplane;
#ifndef USE_DOUBLEBUFFERING
    unsigned long* p0          = (unsigned long*)((unsigned char*)GrayGetPlane(LIGHT_PLANE)+offset);
    unsigned long* p1          = (unsigned long*)((unsigned char*)GrayGetPlane(DARK_PLANE)+offset);
#else
    unsigned long* p0          = (unsigned long*)((unsigned char*)GrayDBufGetHiddenPlane(LIGHT_PLANE)+offset);
    unsigned long* p1          = (unsigned long*)((unsigned char*)GrayDBufGetHiddenPlane(DARK_PLANE)+offset);
#endif
    short          i;

    // NOTE: the lightplane is not taken from the image, but filled
    //       with black (0xFFFFFFFF). otherwise the white text wouldn't
    //       be readable anymore.
    //
    // (The textplane contains white text on black background)

    // loop once for each line to copy ...
    for (i=100;(i--);) {
        *p0++ = *(backplane_0++) & (*tmptext);
        *p1++ = (*tmptext++);
        *p0++ = *(backplane_0++) & (*tmptext);
        *p1++ = (*tmptext++);
        *p0++ = *(backplane_0++) & (*tmptext);
        *p1++ = (*tmptext++);
        *p0++ = *(backplane_0++) & (*tmptext);
        *p1++ = (*tmptext++);
        *p0++ = *(backplane_0++) & (*tmptext);
        *p1++ = (*tmptext++);
        ((unsigned char*)p0)+=10;
        ((unsigned char*)p1)+=10;
    }
#ifdef USE_DOUBLEBUFFERING
    GrayDBufToggleSync();
    if (!_GrayIsRealHW2()) GrayWaitNSwitches(2);
#endif
}


//=============================================================================
// the text which will be scrolled. The number of strings within this array
// will no more be evaluated at compile time.
//=============================================================================
#ifdef USE_DOUBLEBUFFERING

asm("
| 54 items.
    demotext:
      .word string1     - demotext
      .word string0     - demotext
      .word string2     - demotext
      .word string3     - demotext
      .word string4     - demotext
      .word string5     - demotext
      .word string6     - demotext
      .word string7     - demotext
      .word string0     - demotext
      .word string8     - demotext
      .word string0     - demotext
      .word string9     - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string10    - demotext
      .word string11    - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string12    - demotext
      .word string13    - demotext
      .word string14    - demotext
      .word string0     - demotext
      .word string15    - demotext
      .word string16    - demotext
      .word string0     - demotext
      .word string17    - demotext
      .word string18    - demotext
      .word string19    - demotext
      .word string20    - demotext
      .word string21    - demotext
      .word string22    - demotext
      .word string23    - demotext
      .word string24    - demotext
      .word string25    - demotext
      .word string26    - demotext
      .word string27    - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string28    - demotext
      .word string0     - demotext
      .word string29    - demotext
      .word string30    - demotext
      .word string0     - demotext
      .word string31    - demotext
      .word string32    - demotext
      .word string0     - demotext
      .word string33_35 - demotext
      .word string34    - demotext
      .word string33_35 - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string0     - demotext

      
    string0:
      .asciz \"\"
    
    string1:
      .asciz \"TICT-Tutorial S1P6\"

    string2:
      .asciz \"How to write a simple\"

    string3:
      .asciz \"grayscale text scroller\"

    string4:
      .asciz \"with a background image\"

    string5:
      .asciz \"using\"

    string6:
      .asciz \"the TIGCC Environment\"

    string7:
      .asciz \"(double-buffer version).\"

    string8:
      .asciz \"Copyright TICT 2002-2003\"

    string9:
      .asciz \"http://tict.ticalc.org\"
    
    string10:
      .asciz \"Greetings to everyone\"

    string11:
      .asciz \"especially my beta-testers\"
    
    string12:
      .asciz \"You can modify the\"
    
    string13:
      .asciz \"scrolling speed with\"
    
    string14:
      .asciz \"the [+] and [-] keys,\"
    
    string15:
      .asciz \"To pause the scrolling\"
    
    string16:
      .asciz \"press F1.\"

    string17:
      .asciz \"The program synchronizes\"
    
    string18:
      .asciz \"to the Interrupt 1, so it\"
    
    string19:
      .asciz \"is in sync with the\"
    
    string20:
      .asciz \"grayscale routines.\"
    
    string21:
      .asciz \"The grayscale pageflip\"

    string22:
      .asciz \"is done manually when\"
    
    string23:
      .asciz \"the work in both hidden\"
    
    string24:
      .asciz \"planes is finished, so\"
    
    string25:
      .asciz \"there is no phasing in\"
    
    string26:
      .asciz \"this version using\"
    
    string27:
      .asciz \"doublebuffering.\"
    
    string28:
      .asciz \"Have a nice day, Tom.\"
    
    string29:
      .asciz \"Credits for the\"
    
    string30:
      .asciz \"background GFX go to:\"
    
    string31:
      .asciz \"the developers of\"
    
    string32:
      .asciz \"the GENESIS Game\"

    string33_35:
      .asciz \"* * * * *\"
    
    string34:
      .asciz \"Phantasy Star III (TM)\"
");

/*
const char* demotext[] = {
"TICT-Tutorial S1P6","", "How to write a simple", "grayscale text scroller",
"with a background image","using","the TIGCC Environment","(double-buffer version).","",
"Copyright TICT 2002","","http://tict.ticalc.org","","","Greetings to everyone",
"especially my beta-testers","","","You can modify the","scrolling speed with",
"the [+] and [-] keys,","","To pause the scrolling","press F1.","",
"The program synchronizes","to the Interrupt 1, so it","is in sync with the",
"grayscale routines.",
"The grayscale pageflip","is done manually when","the work in both hidden",
"planes is finished, so","there is no phasing in","this version using",
"doublebuffering.","",
"","Have a nice day, Tom","","Credits for the",
"background GFX go to:","","the developers of","the GENESIS Game","","* * * * *",
"Phantasy Star III (TM)","* * * * *","","","","",""};*/
#define SIZE_OF_TEXT 54//(sizeof(demotext)/sizeof(unsigned char *))

#else

asm("
| 61 items.
    demotext:
      .word string1     - demotext
      .word string0     - demotext
      .word string2     - demotext
      .word string3     - demotext
      .word string4     - demotext
      .word string5     - demotext
      .word string6     - demotext
      .word string0     - demotext
      .word string7     - demotext
      .word string0     - demotext
      .word string8     - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string9     - demotext
      .word string10    - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string11    - demotext
      .word string12    - demotext
      .word string13    - demotext
      .word string0     - demotext
      .word string14    - demotext
      .word string15    - demotext
      .word string0     - demotext
      .word string16    - demotext
      .word string17    - demotext
      .word string18    - demotext
      .word string19    - demotext
      .word string20    - demotext
      .word string21    - demotext
      .word string22    - demotext
      .word string23    - demotext
      .word string24    - demotext
      .word string25    - demotext
      .word string26    - demotext
      .word string27    - demotext
      .word string28    - demotext
      .word string29    - demotext
      .word string30    - demotext
      .word string31    - demotext
      .word string32    - demotext
      .word string33    - demotext
      .word string34    - demotext
      .word string35    - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string36    - demotext
      .word string0     - demotext
      .word string37    - demotext
      .word string38    - demotext
      .word string0     - demotext
      .word string39    - demotext
      .word string40    - demotext
      .word string0     - demotext
      .word string41_43 - demotext
      .word string42    - demotext
      .word string41_43 - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string0     - demotext
      .word string0     - demotext

      
    string0:
      .asciz \"\"
    
    string1:
      .asciz \"TICT-Tutorial S1P6\"

    string2:
      .asciz \"How to write a simple\"

    string3:
      .asciz \"grayscale text scroller\"

    string4:
      .asciz \"with a background image\"

    string5:
      .asciz \"using\"

    string6:
      .asciz \"the TIGCC Environment\"

    string7:
      .asciz \"Copyright TICT 2002-2003\"

    string8:
      .asciz \"http://tict.ticalc.org\"
    
    string9:
      .asciz \"Greetings to everyone\"

    string10:
      .asciz \"especially my beta-testers\"
    
    string11:
      .asciz \"You can modify the\"
    
    string12:
      .asciz \"scrolling speed with\"
    
    string13:
      .asciz \"the [+] and [-] keys,\"
    
    string14:
      .asciz \"To pause the scrolling\"
    
    string15:
      .asciz \"press F1.\"

    string16:
      .asciz \"The program synchronizes\"
    
    string17:
      .asciz \"to the Interrupt 1, so it\"
    
    string18:
      .asciz \"is in sync with the\"
    
    string19:
      .asciz \"grayscale routines.\"
    
    string20:
      .asciz \"In this version without\"
    
    string21:
      .asciz \"doublebuffering, the\"
    
    string22:
      .asciz \"grayscale pageflip can\"
    
    string23:
      .asciz \"happen at any time, so\"
    
    string24:
      .asciz \"there might be flicker.\"
    
    string25:
      .asciz \"Additionally the phasing\"
    
    string26:
      .asciz \"effect which is a\"
    
    string27:
      .asciz \"main problem of the\"
      
    string28:
      .asciz \"grayscale support\"
    
    string29:
      .asciz \"might come into play.\"
    
    string30:
      .asciz \"For a short period of\"
    
    string31:
      .asciz \"time the two grayplanes\"

    string32:
      .asciz \"are displayed with a\"
    
    string33:
      .asciz \"line offset of 1, although\"
    
    string34:
      .asciz \"the data is already stored\"
    
    string35:
      .asciz \"in the planes.\"
    
    string36:
      .asciz \"Have a nice day, Tom.\"
    
    string37:
      .asciz \"Credits for the\"
    
    string38:
      .asciz \"background GFX go to:\"
    
    string39:
      .asciz \"the developers of\"
    
    string40:
      .asciz \"the GENESIS Game\"

    string41_43:
      .asciz \"* * * * *\"
    
    string42:
      .asciz \"Phantasy Star III (TM)\"
    
");
/*
const char* demotext[] = {
"TICT-Tutorial S1P6","", "How to write a simple", "grayscale text scroller",
"with a background image","using","the TIGCC Environment","",
"Copyright TICT 2002","","http://tict.ticalc.org","","","Greetings to everyone",
"especially my beta-testers","","","You can modify the","scrolling speed with",
"the [+] and [-] keys,","","To pause the scrolling","press F1.","",
"The program synchronizes","to the Interrupt 1, so it","is in sync with the",
"grayscale routines.",
"In this version without","doublebuffering, the","grayscale pageflip can","happen at any time, so",
"there might be flicker.",
"Additionally the phasing","effect which is a","main problem of the",
"grayscale support","might come into play.","For a short period of",
"time the two grayplanes","are displayed with a",
"line offset of 1, although","the data is already stored","in the planes.","",
"","Have a nice day, Tom","","Credits for the",
"background GFX go to:","","the developers of","the GENESIS Game","","* * * * *",
"Phantasy Star III (tm)","* * * * *","","","","",""};*/
#define SIZE_OF_TEXT 61//(sizeof(demotext)/sizeof(unsigned char *))

#endif

// Old text.
/*
const char* demotext[] = {
"TICT-Tutorial S1P6","", "How to write a simple", "grayscale text scroller",
"with a background image","using","the TIGCC Environment","",
"Copyright TICT 2002","","http://tict.ticalc.org","","","Greetings to everyone",
"especially my beta-testers","","","You can modify the","scrolling speed with",
"the [+] and [-] keys,","","To pause the scrolling","press F1.","",
"I know that the text","flickers.","The program synchronizes",
"to the Interrupt 1, so it","is in principle in sync","with the grayscale",
"routine, but it cannot","synchronize to the","grayscale pageflip",
"frequency, because it","has no access to the","necessary counter.","",
"Additionally the phasing","effect which is a","main problem of the",
"grayscale support","comes into play.","For a short period of",
"time the two grayplanes","are displayed with a",
"line offset of 1, although","the data is already stored","in the planes.","",
"","Have a nice day, Tom","","Credits for the",
"background GFX go to:","","the developers of","the GENESIS Game","","* * * * *",
"Phantasy Star III (tm)","* * * * *","","","","",""};
*/


//=============================================================================
// the scrolling routine itself
//=============================================================================
static inline void ProcessTextScrolling(void) {
    register short          actline = 0;
    register short          i = 9;
    unsigned short          waitnr = 20;
    register unsigned long* src = NULL;
    register unsigned long* dest = NULL;
    short                   loop;

    // fill textplane with black
    memset(S1P6_DATA_ptr->textplane,0xFFFF,20*(100+ADDITIONAL_LINES));

    // clear both grayplanes
    memset(GetPlane(LIGHT_PLANE),0,LCD_SIZE);
    memset(GetPlane(DARK_PLANE),0,LCD_SIZE);

    // loop until the user gets bored ...
    for (;;) {
//        asm("0: bra.s 0b");
        
        asm("
        move.l (%%a2),%1
|        move.l textplane(%%pc),%1
        lea ((LINES_TO_SCROLL*5)*4)(%1),%0
/*
        dest = textplane;
        src  = &textplane[LINES_TO_SCROLL*5];

        // scrolling up the invisible textplane is performed by
        // a kind of memmove (much faster than memmove)...*/
/*
        for (loop = 100+ADDITIONAL_LINES-LINES_TO_SCROLL+1; (loop--);) {
            *dest++ = *src++;
            *dest++ = *src++;
            *dest++ = *src++;
            *dest++ = *src++;
            *dest++ = *src++;
        }
*/
|        asm(
        moveq #100+ADDITIONAL_LINES-LINES_TO_SCROLL-1,%%d0
        _loop_:
        move.l (%0)+,(%1)+
        move.l (%0)+,(%1)+
        move.l (%0)+,(%1)+
        move.l (%0)+,(%1)+
        move.l (%0)+,(%1)+
        dbf %%d0,_loop_
        " : : "a"(src), "a"(dest) : "d0");


        if (i>=7) {
            // fill the line below the last "visible" line with black
            //dest = &textplane[100*5];
            asm ("
            move.l (%%a2),%0
|            move.l textplane(%%pc),%0
            lea 2000(%0),%0
            moveq #-1,%%d0
            move.l %%d0,(%0)+
            move.l %%d0,(%0)+
            move.l %%d0,(%0)+
            move.l %%d0,(%0)+
            move.l %%d0,(%0)+
            " : : "a"(dest) : "d0");
/*            *dest++ = 0xFFFFFFFF;
            *dest++ = 0xFFFFFFFF;
            *dest++ = 0xFFFFFFFF;
            *dest++ = 0xFFFFFFFF;
            *dest++ = 0xFFFFFFFF;*/
        }

        if (i == 9) {
            register unsigned char * str_address asm("%a3");
            unsigned short x = 0;
            
            // it's time to print the next line to the part below the last
            // "visible" line ...
            i = 0;
            memset(&(S1P6_DATA_ptr->textplane[100*5]),0xffff,ADDITIONAL_LINES*20);
            // draw the string centered.


            // Assumes OPTIMIZE_ROM_CALLS is defined...
            asm("
            | a2 (global register variable) mustn't be destroyed !
            pea (%%a2)
            
            move.w %2,%%d0
            add.w %%d0,%%d0
            lea demotext(%%pc),%%a0
            move.w 0(%%a0,%%d0.w),%%a2
            adda.l %%a0,%%a2
            
            pea (%%a2)
            movea.l 0x27E*4(%%a5),%%a0 | strlen
            jsr (%%a0)
            
            addq.l #4,%%sp
            
            add.w %%d0,%%d0
            move.w %%d0,%%d1
            add.w %%d0,%%d0
            add.w %%d1,%%d0
            
            neg.w %%d0
            add.w #160,%%d0
            lsr.w #1,%%d0
            
            move.w %%d0,%1
            movea.l %%a2,%0
            
            move.l (%%sp)+,%%a2
            " : "=a"(str_address), "=d"(x) : "d"(actline) : "d0","d1","d2","a0","a1"/*,"a2"*/);
            XORNormalString(x,str_address);
            //XORNormalString((160-6*strlen(demotext[actline]))>>1,demotext[actline]);
            actline++;
            if (actline == SIZE_OF_TEXT) actline = 0;
        }
        else {
            i += LINES_TO_SCROLL;
        }
      
        // wait a little bit ...
        S1P6_DATA_ptr->counter = 0;
        while (S1P6_DATA_ptr->counter < waitnr);

        // perform a page blit (combines background planes
        // with textplane and copy the result to the grayplanes)
        BlitPage();


        if (OSTimerExpired(APD_TIMER)) off();

        // process a key press...
        if (_keytest(RR_PLUS)) {
            if (waitnr) waitnr -= 2;
            for (loop = 100-1; loop >= 0; loop--) {};
            continue;
        }
        if (_keytest(RR_MINUS)) {
            if (waitnr < 199) waitnr += 2;
            for (loop = 100-1; loop >= 0; loop--) {};
            continue;
        }
        if (_keytest(RR_F1)) {
            // It seems that rebounds are unfrequent if the waiting loop
            // is done that way.
            for (loop = 10000-1; loop >= 0; loop--) {};
            while (_rowread(0x0));
            while (!(_rowread(0x0))) if (OSTimerExpired(APD_TIMER)) off();
            while (_rowread(0x0));
            for (loop = 10000-1; loop >= 0; loop--) {};
        }
        if (_keytest(RR_ESC)) break;
    }
}


//=============================================================================
// where all the fun starts ...
//=============================================================================
void _main(void) {
    S1P6_DATA_ptr = &S1P6_DATA;

    if (!InitScroller()) {
       ST_helpMsg("Error: out of memory !");
       return;
    }
    
    // I want to define CleanupScroller as 'static inline'...
    // Additionnally, the keyboard queue is cleared, which is always a good
    // idea before exit...
    if (!GrayOn()) goto CLEANUP;

// Generates three relocations...
#ifdef USE_DOUBLEBUFFERING
    GrayDBufInit(S1P6_DATA_ptr->dbuffer);
#endif

    ProcessTextScrolling();

    GrayOff();
    
    CLEANUP:

    CleanupScroller();
    
    GKeyFlush();
}
